library IEEE;
use IEEE.std_logic_1164.all;

entity AMUADC_CNTRL is
    port (
        CLK: in STD_LOGIC;
        FSC: in STD_LOGIC;
        RESET: in STD_LOGIC;
        PACKET_DONE: in STD_LOGIC;
        RAW_MODE: in STD_LOGIC;
        EF_ADC_FIFO: in STD_LOGIC;
        SORT_DONE: in STD_LOGIC;
        AZ: out STD_LOGIC;
        CLK_EN: out STD_LOGIC;
        CMP_RST: out STD_LOGIC;
        ADC_DONE: out STD_LOGIC;
        DB_RST: out STD_LOGIC;
        ICS: out STD_LOGIC;
        LOAD: out STD_LOGIC;
        ADC_TIME_OUT: out STD_LOGIC;
        ADC_FIFO_LATCH: out STD_LOGIC;
        ADC_FIFO_RD: out STD_LOGIC;
        AMP_RESET: out STD_LOGIC;
        AMU_RENA: out STD_LOGIC;
        CONNECT: out STD_LOGIC;
        CONV_START: out STD_LOGIC;
        CORR_RST: out STD_LOGIC;
        RADDR_SW: out STD_LOGIC;
        SORT_START: out STD_LOGIC;
        LATCH1: out STD_LOGIC;
        LATCH2: out STD_LOGIC
    );
end AMUADC_CNTRL;

architecture AMUADC_CNTRL_arch of AMUADC_CNTRL is

signal n_value,p_value: STD_LOGIC_VECTOR(5 downto 0):="000000";
signal n_count,p_count: integer range 0 to 63:= 0;
signal ntime_out_count,ptime_out_count: integer range 0 to 256:= 0;
signal pstart,pstart1,start,rst1,load_en,load_en1,load_en2: STD_LOGIC;
signal adc_time_out1,out_count_start,l1,l2,pl1,pl2: STD_LOGIC;
signal out_block1,out_block2, pout_block1,pout_block2: std_logic_vector(7 downto 0);

begin

process (CLK, RESET)
begin
   if RESET='1' then	--asynchronous RESET active High
      p_value <= "000000";
      p_count <= 0;
      pout_block1<="00000000";
      pout_block2<="00000000";
      pl1<='0';
      pl2<='0';
   elsif (CLK'event and CLK='1') then  --CLK rising edge
      p_value <= n_value;
      p_count <= n_count;
      pout_block1<=out_block1;
      pout_block2<=out_block2;
      pl1<=l1;
      pl2<=l2;
   end if;
end process;

process (reset,rst1)
begin
   if rst1='1' then
      load_en1<='0';
   elsif (reset'event and reset='0') then
      load_en1<='1';
   end if;
end process;     

process (rst1,packet_done)
begin
   if rst1='1' then
      load_en2<='0';
   elsif (packet_done'event and packet_done='0') then
      load_en2<='1';
   end if;
end process; 

time_out_clock:process(rst1,clk)
begin
   if rst1='1' then
      ptime_out_count<=0;
   elsif (clk'event and clk='0') then
      ptime_out_count<=ntime_out_count;
   end if;
end process;

time_out_counter:process(ptime_out_count,out_count_start)
begin
   if out_count_start='0' then
      ntime_out_count<=0;
      adc_time_out1<='0';
   elsif ptime_out_count<200 then
      ntime_out_count<=ptime_out_count+1;
      adc_time_out1<='0';
   else
      ntime_out_count<=ptime_out_count;
      adc_time_out1<='1';
   end if; 
end process;     
                                   
load_en<=load_en1 or load_en2;

start<= ef_adc_fifo and not sort_done;

adc_time_out<=adc_time_out1;

adc_fifo_latch<=pout_block1(0);
adc_fifo_rd<=pout_block1(1);
amp_reset<=pout_block1(2);
amu_rena<=pout_block1(3);
connect<=pout_block1(4);
conv_start<=pout_block1(5);
corr_rst<=pout_block1(6);
raddr_sw<=pout_block1(7);

cmp_rst<=pout_block2(0);
ics<=pout_block2(1);
db_rst<=pout_block2(2);
az<=pout_block2(3);
clk_en<=pout_block2(4);
load<=pout_block2(5);
adc_done<=pout_block2(6);
sort_start<=pout_block2(7);

latch1<=pl1;
latch2<=pl2;

process(p_value,p_count,start,raw_mode,fsc,adc_time_out1,load_en)
--start,raw_mode,p_count,load_en,adc_time_out1)
begin
  case p_value is

    when "000000" =>
                   out_block1<="01010100";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_count<=0;
                   if start='1' then                     
                      n_value<="000001";
                   else                        
                      n_value<="000000";
                   end if;
-- Raw mode                      
                       
    when "000001" =>
                   if raw_mode ='1' then
                      n_value<="000011";
                   else                        
                      n_value<="100000";
                   end if;  
                   out_block1<="01010110";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_count<=0;    
    when "000011" =>  
                   out_block1<="01010100";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   if p_count=2 then
                     n_count<=0;
                     n_value<="000010";
                  else   
                     n_count<=p_count+1;
                     n_value<="000011";
                  end if;
    when "000010" =>  
                   out_block1<="01010000";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="000110";
                   n_count<=0;    
    when "000110" =>
                   out_block1<="01011000";
                   out_block2<="00000101";
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   rst1<='0';
                  if p_count=9 then
                     n_count<=0;
                     n_value<="000100";
                  else   
                     n_count<=p_count+1;
                     n_value<="000110";
                  end if;
    when "000100" =>
                   out_block1<="01011000";
                   out_block2<="00000100";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='1';
                   l2<='0';
                   n_value<="000101";
                   n_count<=0;   
    when "000101" =>
                   out_block1<="01011000";
                   out_block2<="00000100";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='1';
                   l2<='0';
                   n_value<="000111";
                   n_count<=0;  
    when "000111" =>
                   out_block1<="01011000";
                   out_block2<="00001100";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='1';
                   l2<='0';
                  if p_count=14 then
                     n_count<=0;
                     n_value<="001111";
                  else   
                     n_count<=p_count+1;
                     n_value<="000111";
                  end if;  
    when "001111" =>
                   out_block1<="01011000";
                   out_block2<="00000100";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                  if p_count=1 then
                     n_count<=0;
                     n_value<="001101";
                  else   
                     n_count<=p_count+1;
                     n_value<="001111";
                  end if;
    when "001101" =>
                   out_block1<="01011000";
                   out_block2<="00000110";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                  if p_count=2 then
                     n_count<=0;
                     n_value<="001100";
                  else   
                     n_count<=p_count+1;
                     n_value<="001101";
                  end if;
    when "001100" =>
                   out_block1<="01010100";
                   out_block2<="00000110";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                  if p_count=1 then
                     n_count<=0;
                     n_value<="001110";
                  else   
                     n_count<=p_count+1;
                     n_value<="001100";
                  end if;
    when "001110" =>
                   out_block1<="01010100";
                   out_block2<="10000010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="001010";
                   n_count<=0;
    when "001010" =>
                   out_block1<="01010100";
                   rst1<='0';
                   l1<='0';
                   l2<='0';
                   if (fsc='1' or adc_time_out1='1') then
                      out_block2<="00000010";
                      n_value<="001011";
                      out_count_start<='1';
                   else
                      out_block2<="00010010";
                      n_value<="001010";
                      out_count_start<='1';
                   end if;   
                   n_count<=0;
   when "001011" =>
                   out_block1<="01010100";
                   out_block2<="00000010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   if load_en='1' then
                      n_value<="001001";
                   else
                      n_value<="001011";
                   end if;   
                   n_count<=0;
   when "001001" =>
                   out_block1<="01010100";
                   out_block2<="00100010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="001000";
                   n_count<=0;
    when "001000" =>
                   out_block1<="01010100";
                   out_block2<="01100010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   if p_count=10 then
                     n_count<=0;
                     n_value<="011000";
                   else   
                     n_count<=p_count+1;
                     n_value<="001000";
                   end if;
    when "011000" =>
                   out_block1<="01010100";
                   out_block2<="01000010";
                   rst1<='1';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="010000";
                   n_count<=0;
    when "010000" =>
                   out_block1<="01010100";
                   out_block2<="01000010";
                   rst1<='1';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="000000";
                   n_count<=0;
                   
-- Correlated mode

    when "100000" =>  
                   out_block1<="11010110";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="100001";
                   n_count<=0;   
    when "100001" =>  
                   out_block1<="11010101";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="100011";
                   n_count<=0; 
    when "100011" =>  
                   out_block1<="11010110";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="100010";
                   n_count<=0;         
    when "100010" =>  
                   if p_count=2 then
                      out_block1<="11010000";
                      out_block2<="00000101";
                      rst1<='0';
                      out_count_start<='0';
                      l1<='0';
                      l2<='0';
                      n_count<=0;
                      n_value<="100110";
                  else 
                      out_block1<="11010000";
                      out_block2<="00000101";
                      rst1<='0';
                      out_count_start<='0';
                      l1<='0';
                      l2<='0';  
                      n_count<=p_count+1;
                      n_value<="100010";
                  end if;
    when "100110" =>  
                   out_block1<="11011000";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="100111";
                   n_count<=0;    
    when "100111" =>
                   out_block1<="11011000";
                   out_block2<="00000101";
                   out_count_start<='0';
                   l1<='1';
                   l2<='0';
                   rst1<='0';
                  if p_count=38 then
                     n_count<=0;
                     n_value<="100101";
                  else   
                     n_count<=p_count+1;
                     n_value<="100111";
                  end if;
    when "100101" =>
                   out_block1<="11001000";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="100100";
                   n_count<=0; 
    when "100100" =>
                   out_block1<="11000100";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="101100";
                   n_count<=0; 
    when "101100" =>
                   out_block1<="01000100";
                   out_block2<="00000101";
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   rst1<='0';
                   if p_count=50 then
                      n_count<=0;
                      n_value<="101101";
                   else   
                      n_count<=p_count+1;
                      n_value<="101100";
                   end if; 
    when "101101" =>
                   out_block1<="01000000";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="101111";
                   n_count<=0;  
    when "101111" =>
                   out_block1<="00001000";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="101110";
                   n_count<=0; 
    when "101110" =>
                   out_block1<="00001000";
                   out_block2<="00000101";
                   out_count_start<='0';
                   l1<='0';
                   l2<='1';
                   rst1<='0';
                   if p_count=60 then
                      n_count<=0;
                      n_value<="101010";
                   else   
                      n_count<=p_count+1;
                      n_value<="101110";
                   end if; 
   when "101010" =>
                   out_block1<="00011000";
                   out_block2<="00001100";
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   rst1<='0';
                   if p_count=3 then
                      n_count<=0;
                      n_value<="101011";
                   else   
                      n_count<=p_count+1;
                      n_value<="101010";
                   end if; 
    when "101011" =>
                   out_block1<="00011000";
                   out_block2<="00001100";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="101001";
                   n_count<=0; 
     when "101001" =>
                   out_block1<="00011000";
                   out_block2<="00001100";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="101000";
                   n_count<=0; 
     when "101000" =>
                   out_block1<="00011000";
                   out_block2<="00000100";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="111000";
                   n_count<=0; 
     when "111000" =>
                   out_block1<="00010100";
                   out_block2<="00000110";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="111001";
                   n_count<=0; 
     when "111001" =>
                   out_block1<="00010100";
                   out_block2<="00010010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="111011";
                   n_count<=0; 
   when "111011" =>
                   out_block1<="00010100";
                   out_block2<="00010010";
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   rst1<='0';
                   if p_count=16 then
                      n_count<=0;
                      n_value<="111010";
                   else   
                      n_count<=p_count+1;
                      n_value<="111011";
                   end if; 
    when "111010" =>
                   out_block1<="10010100";
                   out_block2<="00010010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="111110";
                   n_count<=0; 
    when "111110" =>
                   out_block1<="10010100";
                   rst1<='0';
                   l1<='0';
                   l2<='0';
                   if (fsc='1' or adc_time_out1='1') then
                      out_block2<="00000010";
                      n_value<="111111";
                      out_count_start<='1';
                   else
                      out_block2<="00010010";
                      n_value<="111110";
                      out_count_start<='1';
                   end if;   
                   n_count<=0;
   when "111111" =>
                   out_block1<="10010100";
                   out_block2<="00000010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   if load_en='1' then
                      n_value<="111101";
                   else
                      n_value<="111111";
                   end if;   
                   n_count<=0;
   when "111101" =>
                   out_block1<="10010100";
                   out_block2<="00100010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="111100";
                   n_count<=0;
    when "111100" =>
                   out_block1<="11010100";
                   out_block2<="01100010";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="110100";
                   n_count<=0;      
    when "110100" =>
                   out_block1<="11010100";
                   out_block2<="01000010";
                   rst1<='1';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="110000";
                   n_count<=0;
    when "110000" =>
                   out_block1<="11010100";
                   out_block2<="01000010";
                   rst1<='1';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="000000";
                   n_count<=0; 
                                      
    when others =>
                      
                   out_block1<="00010100";
                   out_block2<="00000101";
                   rst1<='0';
                   out_count_start<='0';
                   l1<='0';
                   l2<='0';
                   n_value<="000000";
                   n_count<=0;
                     
       end case;
    end process;   
end AMUADC_CNTRL_arch;
