--*************************************************************************
-- Forms calibration pulse and lvl-1 for test bench
-- for MVD controller
-- Author:  Melissa C. Smith
--*************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.all;

library SYNOPSYS;
use SYNOPSYS.STD_LOGIC_arith.all;
use SYNOPSYS.STD_LOGIC_unsigned.all;

--*************************************************************************
-- IO Interface Declaration
--*************************************************************************

entity cal_pulse is
port
(
-- inputs

  signal  x_clk:		in  STD_LOGIC;
  signal  beam_clk:        	in  STD_LOGIC;
  signal  reset:	     	in  STD_LOGIC;
  signal  enable:	     	in  STD_LOGIC;
  signal  preamp_rst:		in  STD_LOGIC;

-- outputs
  signal  lvl1_test_pulse: 	out STD_LOGIC:='0';
  signal  cal_pulse:          	out STD_LOGIC:='0';
  signal  end_state:		out STD_LOGIC
);
end cal_pulse;

--*************************************************************************
-- Architecture body
--*************************************************************************

architecture behavior of cal_pulse is

-- Declare constants



-- declare internal signals here

signal nstate, pstate: std_logic_vector(7 downto 0);
signal en: std_logic;

begin

process (beam_clk, reset)
begin
   if reset='1' then	--asynchronous RESET active High
      pstate<=X"00";
   elsif (beam_clk'event and beam_clk='1') then  --CLK rising edge
      pstate <= nstate;
   end if;
end process;

--process (x_clk, reset, end_state)
--begin
--   if (reset='1' or end_state = '1') then	--asynchronous RESET active High
--      en<='0';
--   elsif (x_clk'event and x_clk='1') then  --CLK rising edge
--      if (enable = '1') then
--      	en <= '1';
--      end if;
--   end if;
--end process;

process(pstate,nstate,reset,beam_clk,enable,preamp_rst,enable)

   
begin

   nstate  <= pstate;
   end_state <= '0';
   en <= enable;
  
  if (en='1' and pstate = X"00" and preamp_rst = '1') then
      nstate <= pstate + 1;
  end if;
  
  if (pstate > X"00") then 
	  if (pstate = X"14") then  --count 20 beam ticks and bring cal pulse up
	  	cal_pulse <= '1';
	  	nstate <= pstate +'1';	  
	  elsif (pstate = X"94") then  --count 128 more beam ticks and bring cal pulse down
	  	cal_pulse <= '0';
	  	nstate <= pstate +'1';	  
	  elsif (pstate = X"C0") then  --count 44 more beam ticks and issue lvl1
	  	lvl1_test_pulse <= '1';
	  	nstate <= pstate +'1';	 
	  	end_state <= '1';      -- reset enable signal
	  elsif (pstate = X"C9") then  --off lvl1
	  	lvl1_test_pulse <= '0';	  
	  	nstate <= pstate +'1';
	  	end_state <= '1';	  
	  elsif (pstate = X"CA") then  --reset state machine
	  	nstate <= X"00";
	  	end_state <= '0';
	  else
	  	nstate <= pstate +'1';
	  	end_state <= '0';	  
	  end if;	  
  end if;
end process;
end behavior;




